@@ -1,6 +1,6 @@
+from __future__ import annotations
+
 import os
-from typing import Callable
-from typing import cast
 from typing import List
 from typing import Optional
 from typing import TYPE_CHECKING
@@ -14,6 +14,7 @@ from .script import ScriptDirectory
 if TYPE_CHECKING:
     from alembic.config import Config
     from alembic.script.base import Script
+    from .runtime.environment import ProcessRevisionDirectiveFn
 
 
 def list_templates(config):
@@ -28,7 +29,7 @@ def list_templates(config):
         with open(
             os.path.join(config.get_template_directory(), tempname, "README")
         ) as readme:
-            synopsis = next(readme)
+            synopsis = next(readme).rstrip()
         config.print_stdout("%s - %s", tempname, synopsis)
 
     config.print_stdout("\nTemplates are used via the 'init' command, e.g.:")
@@ -36,7 +37,7 @@ def list_templates(config):
 
 
 def init(
-    config: "Config",
+    config: Config,
     directory: str,
     template: str = "generic",
     package: bool = False,
@@ -86,8 +87,9 @@ def init(
     for file_ in os.listdir(template_dir):
         file_path = os.path.join(template_dir, file_)
         if file_ == "alembic.ini.mako":
-            config_file = os.path.abspath(cast(str, config.config_file_name))
-            if os.access(cast(str, config_file), os.F_OK):
+            assert config.config_file_name is not None
+            config_file = os.path.abspath(config.config_file_name)
+            if os.access(config_file, os.F_OK):
                 util.msg("File %s already exists, skipping" % config_file)
             else:
                 script._generate_template(
@@ -112,7 +114,7 @@ def init(
 
 
 def revision(
-    config: "Config",
+    config: Config,
     message: Optional[str] = None,
     autogenerate: bool = False,
     sql: bool = False,
@@ -122,8 +124,8 @@ def revision(
     version_path: Optional[str] = None,
     rev_id: Optional[str] = None,
     depends_on: Optional[str] = None,
-    process_revision_directives: Callable = None,
-) -> Union[Optional["Script"], List[Optional["Script"]]]:
+    process_revision_directives: Optional[ProcessRevisionDirectiveFn] = None,
+) -> Union[Optional[Script], List[Optional[Script]]]:
     """Create a new revision file.
 
     :param config: a :class:`.Config` object.
@@ -238,13 +240,69 @@ def revision(
         return scripts
 
 
-def merge(
+def check(
     config: "Config",
+) -> None:
+    """Check if revision command with autogenerate has pending upgrade ops.
+
+    :param config: a :class:`.Config` object.
+
+    .. versionadded:: 1.9.0
+
+    """
+
+    script_directory = ScriptDirectory.from_config(config)
+
+    command_args = dict(
+        message=None,
+        autogenerate=True,
+        sql=False,
+        head="head",
+        splice=False,
+        branch_label=None,
+        version_path=None,
+        rev_id=None,
+        depends_on=None,
+    )
+    revision_context = autogen.RevisionContext(
+        config,
+        script_directory,
+        command_args,
+    )
+
+    def retrieve_migrations(rev, context):
+        revision_context.run_autogenerate(rev, context)
+        return []
+
+    with EnvironmentContext(
+        config,
+        script_directory,
+        fn=retrieve_migrations,
+        as_sql=False,
+        template_args=revision_context.template_args,
+        revision_context=revision_context,
+    ):
+        script_directory.run_env()
+
+    # the revision_context now has MigrationScript structure(s) present.
+
+    migration_script = revision_context.generated_revisions[-1]
+    diffs = migration_script.upgrade_ops.as_diffs()
+    if diffs:
+        raise util.AutogenerateDiffsDetected(
+            f"New upgrade operations detected: {diffs}"
+        )
+    else:
+        config.print_stdout("No new upgrade operations detected.")
+
+
+def merge(
+    config: Config,
     revisions: str,
-    message: str = None,
-    branch_label: str = None,
-    rev_id: str = None,
-) -> Optional["Script"]:
+    message: Optional[str] = None,
+    branch_label: Optional[str] = None,
+    rev_id: Optional[str] = None,
+) -> Optional[Script]:
     """Merge two revisions together.  Creates a new migration file.
 
     :param config: a :class:`.Config` instance
@@ -264,7 +322,7 @@ def merge(
 
     script = ScriptDirectory.from_config(config)
     template_args = {
-        "config": "config"  # Let templates use config for
+        "config": config  # Let templates use config for
         # e.g. multiple databases
     }
     return script.generate_revision(
@@ -273,12 +331,12 @@ def merge(
         refresh=True,
         head=revisions,
         branch_labels=branch_label,
-        **template_args  # type:ignore[arg-type]
+        **template_args,  # type:ignore[arg-type]
     )
 
 
 def upgrade(
-    config: "Config",
+    config: Config,
     revision: str,
     sql: bool = False,
     tag: Optional[str] = None,
@@ -321,7 +379,7 @@ def upgrade(
 
 
 def downgrade(
-    config: "Config",
+    config: Config,
     revision: str,
     sql: bool = False,
     tag: Optional[str] = None,
@@ -392,7 +450,7 @@ def show(config, rev):
 
 
 def history(
-    config: "Config",
+    config: Config,
     rev_range: Optional[str] = None,
     verbose: bool = False,
     indicate_current: bool = False,
@@ -515,7 +573,7 @@ def branches(config, verbose=False):
             )
 
 
-def current(config: "Config", verbose: bool = False) -> None:
+def current(config: Config, verbose: bool = False) -> None:
     """Display the current revision for a database.
 
     :param config: a :class:`.Config` instance.
@@ -544,7 +602,7 @@ def current(config: "Config", verbose: bool = False) -> None:
 
 
 def stamp(
-    config: "Config",
+    config: Config,
     revision: str,
     sql: bool = False,
     tag: Optional[str] = None,
@@ -613,7 +671,7 @@ def stamp(
         script.run_env()
 
 
-def edit(config: "Config", rev: str) -> None:
+def edit(config: Config, rev: str) -> None:
     """Edit revision script(s) using $EDITOR.
 
     :param config: a :class:`.Config` instance.
@@ -642,4 +700,31 @@ def edit(config: "Config", rev: str) -> None:
                 "No revision files indicated by symbol '%s'" % rev
             )
         for sc in revs:
+            assert sc
             util.open_in_editor(sc.path)
+
+
+def ensure_version(config: Config, sql: bool = False) -> None:
+    """Create the alembic version table if it doesn't exist already .
+
+    :param config: a :class:`.Config` instance.
+
+    :param sql: use ``--sql`` mode
+
+     .. versionadded:: 1.7.6
+
+    """
+
+    script = ScriptDirectory.from_config(config)
+
+    def do_ensure_version(rev, context):
+        context._ensure_version_table()
+        return []
+
+    with EnvironmentContext(
+        config,
+        script,
+        fn=do_ensure_version,
+        as_sql=sql,
+    ):
+        script.run_env()
