@@ -1,21 +1,48 @@
+from __future__ import annotations
+
 from contextlib import contextmanager
+import re
 import textwrap
+from typing import Any
+from typing import Callable
+from typing import Dict
+from typing import Iterator
+from typing import List  # noqa
+from typing import Mapping
+from typing import Optional
+from typing import Sequence  # noqa
+from typing import Tuple
+from typing import Type  # noqa
+from typing import TYPE_CHECKING
+from typing import Union
+
+from sqlalchemy.sql.elements import conv
 
 from . import batch
 from . import schemaobj
 from .. import util
 from ..util import sqla_compat
-from ..util.compat import exec_
+from ..util.compat import formatannotation_fwdref
 from ..util.compat import inspect_formatargspec
-from ..util.compat import inspect_getargspec
+from ..util.compat import inspect_getfullargspec
+from ..util.sqla_compat import _literal_bindparam
+
+
+NoneType = type(None)
+
+if TYPE_CHECKING:
+    from typing import Literal
+
+    from sqlalchemy import Table  # noqa
+    from sqlalchemy.engine import Connection
+
+    from .batch import BatchOperationsImpl
+    from .ops import MigrateOperation
+    from ..ddl import DefaultImpl
+    from ..runtime.migration import MigrationContext
 
 __all__ = ("Operations", "BatchOperations")
 
-try:
-    from sqlalchemy.sql.naming import conv
-except:
-    conv = None
-
 
 class Operations(util.ModuleClsProxy):
 
@@ -48,9 +75,14 @@ class Operations(util.ModuleClsProxy):
 
     """
 
+    impl: Union[DefaultImpl, BatchOperationsImpl]
     _to_impl = util.Dispatcher()
 
-    def __init__(self, migration_context, impl=None):
+    def __init__(
+        self,
+        migration_context: MigrationContext,
+        impl: Optional[BatchOperationsImpl] = None,
+    ) -> None:
         """Construct a new :class:`.Operations`
 
         :param migration_context: a :class:`.MigrationContext`
@@ -66,7 +98,9 @@ class Operations(util.ModuleClsProxy):
         self.schema_obj = schemaobj.SchemaObjects(migration_context)
 
     @classmethod
-    def register_operation(cls, name, sourcename=None):
+    def register_operation(
+        cls, name: str, sourcename: Optional[str] = None
+    ) -> Callable[..., Any]:
         """Register a new operation for this class.
 
         This method is normally used to add new operations
@@ -91,14 +125,16 @@ class Operations(util.ModuleClsProxy):
                 fn = getattr(op_cls, sourcename)
                 source_name = fn.__name__
 
-            spec = inspect_getargspec(fn)
+            spec = inspect_getfullargspec(fn)
 
             name_args = spec[0]
             assert name_args[0:2] == ["cls", "operations"]
 
             name_args[0:2] = ["self"]
 
-            args = inspect_formatargspec(*spec)
+            args = inspect_formatargspec(
+                *spec, formatannotation=formatannotation_fwdref
+            )
             num_defaults = len(spec[3]) if spec[3] else 0
             if num_defaults:
                 defaulted_vals = name_args[0 - num_defaults :]
@@ -111,6 +147,13 @@ class Operations(util.ModuleClsProxy):
                 spec[2],
                 defaulted_vals,
                 formatvalue=lambda x: "=" + x,
+                formatannotation=formatannotation_fwdref,
+            )
+
+            args = re.sub(
+                r'[_]?ForwardRef\(([\'"].+?[\'"])\)',
+                lambda m: m.group(1),
+                args,
             )
 
             func_text = textwrap.dedent(
@@ -127,9 +170,11 @@ class Operations(util.ModuleClsProxy):
                     "doc": fn.__doc__,
                 }
             )
-            globals_ = {"op_cls": op_cls}
+            globals_ = dict(globals())
+            globals_.update({"op_cls": op_cls})
             lcl = {}
-            exec_(func_text, globals_, lcl)
+
+            exec(func_text, globals_, lcl)
             setattr(cls, name, lcl[name])
             fn.__func__.__doc__ = (
                 "This method is proxied on "
@@ -143,7 +188,7 @@ class Operations(util.ModuleClsProxy):
         return register
 
     @classmethod
-    def implementation_for(cls, op_cls):
+    def implementation_for(cls, op_cls: Any) -> Callable[..., Any]:
         """Register an implementation for a given :class:`.MigrateOperation`.
 
         This is part of the operation extensibility API.
@@ -162,7 +207,9 @@ class Operations(util.ModuleClsProxy):
 
     @classmethod
     @contextmanager
-    def context(cls, migration_context):
+    def context(
+        cls, migration_context: MigrationContext
+    ) -> Iterator[Operations]:
         op = Operations(migration_context)
         op._install_proxy()
         yield op
@@ -171,17 +218,17 @@ class Operations(util.ModuleClsProxy):
     @contextmanager
     def batch_alter_table(
         self,
-        table_name,
-        schema=None,
-        recreate="auto",
-        partial_reordering=None,
-        copy_from=None,
-        table_args=(),
-        table_kwargs=util.immutabledict(),
-        reflect_args=(),
-        reflect_kwargs=util.immutabledict(),
-        naming_convention=None,
-    ):
+        table_name: str,
+        schema: Optional[str] = None,
+        recreate: Literal["auto", "always", "never"] = "auto",
+        partial_reordering: Optional[tuple] = None,
+        copy_from: Optional[Table] = None,
+        table_args: Tuple[Any, ...] = (),
+        table_kwargs: Mapping[str, Any] = util.immutabledict(),
+        reflect_args: Tuple[Any, ...] = (),
+        reflect_kwargs: Mapping[str, Any] = util.immutabledict(),
+        naming_convention: Optional[Dict[str, str]] = None,
+    ) -> Iterator[BatchOperations]:
         """Invoke a series of per-table migrations in batch.
 
         Batch mode allows a series of operations specific to a table
@@ -335,7 +382,7 @@ class Operations(util.ModuleClsProxy):
         yield batch_op
         impl.flush()
 
-    def get_context(self):
+    def get_context(self) -> MigrationContext:
         """Return the :class:`.MigrationContext` object that's
         currently in use.
 
@@ -343,7 +390,7 @@ class Operations(util.ModuleClsProxy):
 
         return self.migration_context
 
-    def invoke(self, operation):
+    def invoke(self, operation: MigrateOperation) -> Any:
         """Given a :class:`.MigrateOperation`, invoke it in terms of
         this :class:`.Operations` instance.
 
@@ -353,7 +400,7 @@ class Operations(util.ModuleClsProxy):
         )
         return fn(self, operation)
 
-    def f(self, name):
+    def f(self, name: str) -> conv:
         """Indicate a string name that has already had a naming convention
         applied to it.
 
@@ -386,20 +433,14 @@ class Operations(util.ModuleClsProxy):
             CONSTRAINT ck_bool_t_x CHECK (x in (1, 0)))
 
         The function is rendered in the output of autogenerate when
-        a particular constraint name is already converted, for SQLAlchemy
-        version **0.9.4 and greater only**.   Even though ``naming_convention``
-        was introduced in 0.9.2, the string disambiguation service is new
-        as of 0.9.4.
+        a particular constraint name is already converted.
 
         """
-        if conv:
-            return conv(name)
-        else:
-            raise NotImplementedError(
-                "op.f() feature requires SQLAlchemy 0.9.4 or greater."
-            )
+        return conv(name)
 
-    def inline_literal(self, value, type_=None):
+    def inline_literal(
+        self, value: Union[str, int], type_: None = None
+    ) -> _literal_bindparam:
         r"""Produce an 'inline literal' expression, suitable for
         using in an INSERT, UPDATE, or DELETE statement.
 
@@ -443,7 +484,7 @@ class Operations(util.ModuleClsProxy):
         """
         return sqla_compat._literal_bindparam(None, value, type_=type_)
 
-    def get_bind(self):
+    def get_bind(self) -> Connection:
         """Return the current 'bind'.
 
         Under normal circumstances, this is the
@@ -453,7 +494,7 @@ class Operations(util.ModuleClsProxy):
         In a SQL script context, this value is ``None``. [TODO: verify this]
 
         """
-        return self.migration_context.impl.bind
+        return self.migration_context.impl.bind  # type: ignore[return-value]
 
 
 class BatchOperations(Operations):
@@ -473,6 +514,8 @@ class BatchOperations(Operations):
 
     """
 
+    impl: BatchOperationsImpl
+
     def _noop(self, operation):
         raise NotImplementedError(
             "The %s method does not apply to a batch table alter operation."
