@@ -1,5 +1,6 @@
+from __future__ import annotations
+
 from typing import Any
-from typing import cast
 from typing import Dict
 from typing import List
 from typing import Optional
@@ -23,8 +24,10 @@ from sqlalchemy.util import topological
 from ..util import exc
 from ..util.sqla_compat import _columns_for_constraint
 from ..util.sqla_compat import _copy
+from ..util.sqla_compat import _copy_expression
 from ..util.sqla_compat import _ensure_scope_for_ddl
 from ..util.sqla_compat import _fk_is_self_referential
+from ..util.sqla_compat import _idx_table_bound_expressions
 from ..util.sqla_compat import _insert_inline
 from ..util.sqla_compat import _is_type_bound
 from ..util.sqla_compat import _remove_column_from_collection
@@ -83,11 +86,11 @@ class BatchOperationsImpl:
         self.batch = []
 
     @property
-    def dialect(self) -> "Dialect":
+    def dialect(self) -> Dialect:
         return self.operations.impl.dialect
 
     @property
-    def impl(self) -> "DefaultImpl":
+    def impl(self) -> DefaultImpl:
         return self.operations.impl
 
     def _should_recreate(self) -> bool:
@@ -116,13 +119,28 @@ class BatchOperationsImpl:
                     existing_table = self.copy_from
                     reflected = False
                 else:
+                    if self.operations.migration_context.as_sql:
+                        raise exc.CommandError(
+                            f"This operation cannot proceed in --sql mode; "
+                            f"batch mode with dialect "
+                            f"{self.operations.migration_context.dialect.name} "  # noqa: E501
+                            f"requires a live database connection with which "
+                            f'to reflect the table "{self.table_name}". '
+                            f"To generate a batch SQL migration script using "
+                            "table "
+                            '"move and copy", a complete Table object '
+                            f'should be passed to the "copy_from" argument '
+                            "of the batch_alter_table() method so that table "
+                            "reflection can be skipped."
+                        )
+
                     existing_table = Table(
                         self.table_name,
                         m1,
                         schema=self.schema,
                         autoload_with=self.operations.get_bind(),
                         *self.reflect_args,
-                        **self.reflect_kwargs
+                        **self.reflect_kwargs,
                     )
                     reflected = True
 
@@ -156,19 +174,19 @@ class BatchOperationsImpl:
     def drop_column(self, *arg, **kw) -> None:
         self.batch.append(("drop_column", arg, kw))
 
-    def add_constraint(self, const: "Constraint") -> None:
+    def add_constraint(self, const: Constraint) -> None:
         self.batch.append(("add_constraint", (const,), {}))
 
-    def drop_constraint(self, const: "Constraint") -> None:
+    def drop_constraint(self, const: Constraint) -> None:
         self.batch.append(("drop_constraint", (const,), {}))
 
     def rename_table(self, *arg, **kw):
         self.batch.append(("rename_table", arg, kw))
 
-    def create_index(self, idx: "Index") -> None:
+    def create_index(self, idx: Index) -> None:
         self.batch.append(("create_index", (idx,), {}))
 
-    def drop_index(self, idx: "Index") -> None:
+    def drop_index(self, idx: Index) -> None:
         self.batch.append(("drop_index", (idx,), {}))
 
     def create_table_comment(self, table):
@@ -190,8 +208,8 @@ class BatchOperationsImpl:
 class ApplyBatchImpl:
     def __init__(
         self,
-        impl: "DefaultImpl",
-        table: "Table",
+        impl: DefaultImpl,
+        table: Table,
         table_args: tuple,
         table_kwargs: Dict[str, Any],
         reflected: bool,
@@ -218,12 +236,12 @@ class ApplyBatchImpl:
         self._grab_table_elements()
 
     @classmethod
-    def _calc_temp_name(cls, tablename: "quoted_name") -> str:
+    def _calc_temp_name(cls, tablename: Union[quoted_name, str]) -> str:
         return ("_alembic_tmp_%s" % tablename)[0:50]
 
     def _grab_table_elements(self) -> None:
         schema = self.table.schema
-        self.columns: Dict[str, "Column"] = OrderedDict()
+        self.columns: Dict[str, Column] = OrderedDict()
         for c in self.table.c:
             c_copy = _copy(c, schema=schema)
             c_copy.unique = c_copy.index = False
@@ -232,11 +250,11 @@ class ApplyBatchImpl:
             if isinstance(c.type, SchemaEventTarget):
                 assert c_copy.type is not c.type
             self.columns[c.name] = c_copy
-        self.named_constraints: Dict[str, "Constraint"] = {}
+        self.named_constraints: Dict[str, Constraint] = {}
         self.unnamed_constraints = []
         self.col_named_constraints = {}
-        self.indexes: Dict[str, "Index"] = {}
-        self.new_indexes: Dict[str, "Index"] = {}
+        self.indexes: Dict[str, Index] = {}
+        self.new_indexes: Dict[str, Index] = {}
 
         for const in self.table.constraints:
             if _is_type_bound(const):
@@ -262,7 +280,7 @@ class ApplyBatchImpl:
                         self.col_named_constraints[const.name] = (col, const)
 
         for idx in self.table.indexes:
-            self.indexes[idx.name] = idx
+            self.indexes[idx.name] = idx  # type: ignore[index]
 
         for k in self.table.kwargs:
             self.table_kwargs.setdefault(k, self.table.kwargs[k])
@@ -311,21 +329,19 @@ class ApplyBatchImpl:
             m,
             *(list(self.columns.values()) + list(self.table_args)),
             schema=schema,
-            **self.table_kwargs
+            **self.table_kwargs,
         )
 
         for const in (
             list(self.named_constraints.values()) + self.unnamed_constraints
         ):
 
-            const_columns = set(
-                [c.key for c in _columns_for_constraint(const)]
-            )
+            const_columns = {c.key for c in _columns_for_constraint(const)}
 
             if not const_columns.issubset(self.column_transfers):
                 continue
 
-            const_copy: "Constraint"
+            const_copy: Constraint
             if isinstance(const, ForeignKeyConstraint):
                 if _fk_is_self_referential(const):
                     # for self-referential constraint, refer to the
@@ -350,23 +366,41 @@ class ApplyBatchImpl:
                 self._setup_referent(m, const)
             new_table.append_constraint(const_copy)
 
-    def _gather_indexes_from_both_tables(self) -> List["Index"]:
+    def _gather_indexes_from_both_tables(self) -> List[Index]:
         assert self.new_table is not None
         idx: List[Index] = []
-        idx.extend(self.indexes.values())
+
+        for idx_existing in self.indexes.values():
+            # this is a lift-and-move from Table.to_metadata
+
+            if idx_existing._column_flag:  # type: ignore
+                continue
+
+            idx_copy = Index(
+                idx_existing.name,
+                unique=idx_existing.unique,
+                *[
+                    _copy_expression(expr, self.new_table)
+                    for expr in _idx_table_bound_expressions(idx_existing)
+                ],
+                _table=self.new_table,
+                **idx_existing.kwargs,
+            )
+            idx.append(idx_copy)
+
         for index in self.new_indexes.values():
             idx.append(
                 Index(
                     index.name,
                     unique=index.unique,
                     *[self.new_table.c[col] for col in index.columns.keys()],
-                    **index.kwargs
+                    **index.kwargs,
                 )
             )
         return idx
 
     def _setup_referent(
-        self, metadata: "MetaData", constraint: "ForeignKeyConstraint"
+        self, metadata: MetaData, constraint: ForeignKeyConstraint
     ) -> None:
         spec = constraint.elements[
             0
@@ -401,10 +435,10 @@ class ApplyBatchImpl:
                             for elem in constraint.elements
                         ]
                     ],
-                    schema=referent_schema
+                    schema=referent_schema,
                 )
 
-    def _create(self, op_impl: "DefaultImpl") -> None:
+    def _create(self, op_impl: DefaultImpl) -> None:
         self._transfer_elements_to_new_table()
 
         op_impl.prep_table_for_batch(self, self.table)
@@ -448,12 +482,12 @@ class ApplyBatchImpl:
         table_name: str,
         column_name: str,
         nullable: Optional[bool] = None,
-        server_default: Optional[Union["Function", str, bool]] = False,
+        server_default: Optional[Union[Function, str, bool]] = False,
         name: Optional[str] = None,
-        type_: Optional["TypeEngine"] = None,
+        type_: Optional[TypeEngine] = None,
         autoincrement: None = None,
-        comment: Union[str, "Literal[False]"] = False,
-        **kw
+        comment: Union[str, Literal[False]] = False,
+        **kw,
     ) -> None:
         existing = self.columns[column_name]
         existing_transfer: Dict[str, Any] = self.column_transfers[column_name]
@@ -475,8 +509,8 @@ class ApplyBatchImpl:
                     and resolved_existing_type.name  # type:ignore[attr-defined]  # noqa E501
                 ):
                     self.named_constraints.pop(
-                        resolved_existing_type.name,
-                        None,  # type:ignore[attr-defined]
+                        resolved_existing_type.name,  # type:ignore[attr-defined]  # noqa E501
+                        None,
                     )
 
         if type_ is not None:
@@ -510,7 +544,7 @@ class ApplyBatchImpl:
                 existing.server_default = None
             else:
                 sql_schema.DefaultClause(
-                    server_default
+                    server_default  # type: ignore[arg-type]
                 )._set_parent(  # type:ignore[attr-defined]
                     existing
                 )
@@ -551,9 +585,9 @@ class ApplyBatchImpl:
                             insert_after = index_cols[idx]
                     else:
                         # insert before a column that is also new
-                        insert_after = dict(
-                            (b, a) for a, b in self.add_col_ordering
-                        )[insert_before]
+                        insert_after = {
+                            b: a for a, b in self.add_col_ordering
+                        }[insert_before]
 
         if insert_before:
             self.add_col_ordering += ((colname, insert_before),)
@@ -571,10 +605,10 @@ class ApplyBatchImpl:
     def add_column(
         self,
         table_name: str,
-        column: "Column",
+        column: Column,
         insert_before: Optional[str] = None,
         insert_after: Optional[str] = None,
-        **kw
+        **kw,
     ) -> None:
         self._setup_dependencies_for_add_column(
             column.name, insert_before, insert_after
@@ -585,7 +619,7 @@ class ApplyBatchImpl:
         self.column_transfers[column.name] = {}
 
     def drop_column(
-        self, table_name: str, column: Union["ColumnClause", "Column"], **kw
+        self, table_name: str, column: Union[ColumnClause, Column], **kw
     ) -> None:
         if column.name in self.table.primary_key.columns:
             _remove_column_from_collection(
@@ -627,7 +661,7 @@ class ApplyBatchImpl:
 
         """
 
-    def add_constraint(self, const: "Constraint") -> None:
+    def add_constraint(self, const: Constraint) -> None:
         if not const.name:
             raise ValueError("Constraint must have a name")
         if isinstance(const, sql_schema.PrimaryKeyConstraint):
@@ -636,7 +670,7 @@ class ApplyBatchImpl:
 
         self.named_constraints[const.name] = const
 
-    def drop_constraint(self, const: "Constraint") -> None:
+    def drop_constraint(self, const: Constraint) -> None:
         if not const.name:
             raise ValueError("Constraint must have a name")
         try:
@@ -647,7 +681,8 @@ class ApplyBatchImpl:
                     if col_const.name == const.name:
                         self.columns[col.name].constraints.remove(col_const)
             else:
-                const = self.named_constraints.pop(cast(str, const.name))
+                assert const.name
+                const = self.named_constraints.pop(const.name)
         except KeyError:
             if _is_type_bound(const):
                 # type-bound constraints are only included in the new
@@ -661,12 +696,12 @@ class ApplyBatchImpl:
                 for col in const.columns:
                     self.columns[col.name].primary_key = False
 
-    def create_index(self, idx: "Index") -> None:
-        self.new_indexes[idx.name] = idx
+    def create_index(self, idx: Index) -> None:
+        self.new_indexes[idx.name] = idx  # type: ignore[index]
 
-    def drop_index(self, idx: "Index") -> None:
+    def drop_index(self, idx: Index) -> None:
         try:
-            del self.indexes[idx.name]
+            del self.indexes[idx.name]  # type: ignore[arg-type]
         except KeyError:
             raise ValueError("No such index: '%s'" % idx.name)
 
