@@ -1,7 +1,40 @@
+from __future__ import annotations
+
+from typing import Any
+from typing import Callable
+from typing import ContextManager
+from typing import Dict
+from typing import List
+from typing import Optional
+from typing import overload
+from typing import TextIO
+from typing import Tuple
+from typing import TYPE_CHECKING
+from typing import Union
+
+from .migration import _ProxyTransaction
 from .migration import MigrationContext
 from .. import util
 from ..operations import Operations
 
+if TYPE_CHECKING:
+    from typing import Literal
+
+    from sqlalchemy.engine.base import Connection
+    from sqlalchemy.sql.elements import ClauseElement
+    from sqlalchemy.sql.schema import MetaData
+
+    from ..config import Config
+    from ..ddl import DefaultImpl
+    from ..operations.ops import MigrateOperation
+    from ..script.base import ScriptDirectory
+
+_RevNumber = Optional[Union[str, Tuple[str, ...]]]
+
+ProcessRevisionDirectiveFn = Callable[
+    [MigrationContext, Tuple[str, str], List["MigrateOperation"]], None
+]
+
 
 class EnvironmentContext(util.ModuleClsProxy):
 
@@ -66,21 +99,23 @@ class EnvironmentContext(util.ModuleClsProxy):
 
     """
 
-    _migration_context = None
+    _migration_context: Optional[MigrationContext] = None
 
-    config = None
+    config: Config = None  # type:ignore[assignment]
     """An instance of :class:`.Config` representing the
     configuration file contents as well as other variables
     set programmatically within it."""
 
-    script = None
+    script: ScriptDirectory = None  # type:ignore[assignment]
     """An instance of :class:`.ScriptDirectory` which provides
     programmatic access to version files within the ``versions/``
     directory.
 
     """
 
-    def __init__(self, config, script, **kw):
+    def __init__(
+        self, config: Config, script: ScriptDirectory, **kw: Any
+    ) -> None:
         r"""Construct a new :class:`.EnvironmentContext`.
 
         :param config: a :class:`.Config` instance.
@@ -94,7 +129,7 @@ class EnvironmentContext(util.ModuleClsProxy):
         self.script = script
         self.context_opts = kw
 
-    def __enter__(self):
+    def __enter__(self) -> EnvironmentContext:
         """Establish a context which provides a
         :class:`.EnvironmentContext` object to
         env.py scripts.
@@ -106,15 +141,15 @@ class EnvironmentContext(util.ModuleClsProxy):
         self._install_proxy()
         return self
 
-    def __exit__(self, *arg, **kw):
+    def __exit__(self, *arg: Any, **kw: Any) -> None:
         self._remove_proxy()
 
-    def is_offline_mode(self):
+    def is_offline_mode(self) -> bool:
         """Return True if the current migrations environment
         is running in "offline mode".
 
         This is ``True`` or ``False`` depending
-        on the the ``--sql`` flag passed.
+        on the ``--sql`` flag passed.
 
         This function does not require that the :class:`.MigrationContext`
         has been configured.
@@ -136,10 +171,10 @@ class EnvironmentContext(util.ModuleClsProxy):
         """
         return self.get_context().impl.transactional_ddl
 
-    def requires_connection(self):
+    def requires_connection(self) -> bool:
         return not self.is_offline_mode()
 
-    def get_head_revision(self):
+    def get_head_revision(self) -> _RevNumber:
         """Return the hex identifier of the 'head' script revision.
 
         If the script directory has multiple heads, this
@@ -154,7 +189,7 @@ class EnvironmentContext(util.ModuleClsProxy):
         """
         return self.script.as_revision_number("head")
 
-    def get_head_revisions(self):
+    def get_head_revisions(self) -> _RevNumber:
         """Return the hex identifier of the 'heads' script revision(s).
 
         This returns a tuple containing the version number of all
@@ -163,12 +198,10 @@ class EnvironmentContext(util.ModuleClsProxy):
         This function does not require that the :class:`.MigrationContext`
         has been configured.
 
-        .. versionadded:: 0.7.0
-
         """
         return self.script.as_revision_number("heads")
 
-    def get_starting_revision_argument(self):
+    def get_starting_revision_argument(self) -> _RevNumber:
         """Return the 'starting revision' argument,
         if the revision was passed using ``start:end``.
 
@@ -197,7 +230,7 @@ class EnvironmentContext(util.ModuleClsProxy):
                 "No starting revision argument is available."
             )
 
-    def get_revision_argument(self):
+    def get_revision_argument(self) -> _RevNumber:
         """Get the 'destination' revision argument.
 
         This is typically the argument passed to the
@@ -215,7 +248,7 @@ class EnvironmentContext(util.ModuleClsProxy):
             self.context_opts["destination_rev"]
         )
 
-    def get_tag_argument(self):
+    def get_tag_argument(self) -> Optional[str]:
         """Return the value passed for the ``--tag`` argument, if any.
 
         The ``--tag`` argument is not used directly by Alembic,
@@ -235,7 +268,23 @@ class EnvironmentContext(util.ModuleClsProxy):
         """
         return self.context_opts.get("tag", None)
 
-    def get_x_argument(self, as_dictionary=False):
+    @overload
+    def get_x_argument(self, as_dictionary: Literal[False]) -> List[str]:
+        ...
+
+    @overload
+    def get_x_argument(self, as_dictionary: Literal[True]) -> Dict[str, str]:
+        ...
+
+    @overload
+    def get_x_argument(
+        self, as_dictionary: bool = ...
+    ) -> Union[List[str], Dict[str, str]]:
+        ...
+
+    def get_x_argument(
+        self, as_dictionary: bool = False
+    ) -> Union[List[str], Dict[str, str]]:
         """Return the value(s) passed for the ``-x`` argument, if any.
 
         The ``-x`` argument is an open ended flag that allows any user-defined
@@ -267,8 +316,6 @@ class EnvironmentContext(util.ModuleClsProxy):
         This function does not require that the :class:`.MigrationContext`
         has been configured.
 
-        .. versionadded:: 0.6.0
-
         .. seealso::
 
             :meth:`.EnvironmentContext.get_tag_argument`
@@ -286,34 +333,36 @@ class EnvironmentContext(util.ModuleClsProxy):
 
     def configure(
         self,
-        connection=None,
-        url=None,
-        dialect_name=None,
-        dialect_opts=None,
-        transactional_ddl=None,
-        transaction_per_migration=False,
-        output_buffer=None,
-        starting_rev=None,
-        tag=None,
-        template_args=None,
-        render_as_batch=False,
-        target_metadata=None,
-        include_symbol=None,
-        include_object=None,
-        include_schemas=False,
-        process_revision_directives=None,
-        compare_type=False,
-        compare_server_default=False,
-        render_item=None,
-        literal_binds=False,
-        upgrade_token="upgrades",
-        downgrade_token="downgrades",
-        alembic_module_prefix="op.",
-        sqlalchemy_module_prefix="sa.",
-        user_module_prefix=None,
-        on_version_apply=None,
-        **kw
-    ):
+        connection: Optional[Connection] = None,
+        url: Optional[str] = None,
+        dialect_name: Optional[str] = None,
+        dialect_opts: Optional[Dict[str, Any]] = None,
+        transactional_ddl: Optional[bool] = None,
+        transaction_per_migration: bool = False,
+        output_buffer: Optional[TextIO] = None,
+        starting_rev: Optional[str] = None,
+        tag: Optional[str] = None,
+        template_args: Optional[Dict[str, Any]] = None,
+        render_as_batch: bool = False,
+        target_metadata: Optional[MetaData] = None,
+        include_name: Optional[Callable[..., bool]] = None,
+        include_object: Optional[Callable[..., bool]] = None,
+        include_schemas: bool = False,
+        process_revision_directives: Optional[
+            ProcessRevisionDirectiveFn
+        ] = None,
+        compare_type: bool = False,
+        compare_server_default: bool = False,
+        render_item: Optional[Callable[..., bool]] = None,
+        literal_binds: bool = False,
+        upgrade_token: str = "upgrades",
+        downgrade_token: str = "downgrades",
+        alembic_module_prefix: str = "op.",
+        sqlalchemy_module_prefix: str = "sa.",
+        user_module_prefix: Optional[str] = None,
+        on_version_apply: Optional[Callable[..., None]] = None,
+        **kw: Any,
+    ) -> None:
         """Configure a :class:`.MigrationContext` within this
         :class:`.EnvironmentContext` which will provide database
         connectivity and other configuration to a series of
@@ -368,9 +417,6 @@ class EnvironmentContext(util.ModuleClsProxy):
         :param transaction_per_migration: if True, nest each migration script
          in a transaction rather than the full series of migrations to
          run.
-
-         .. versionadded:: 0.6.5
-
         :param output_buffer: a file-like object that will be used
          for textual output
          when the ``--sql`` option is used to generate SQL scripts.
@@ -394,8 +440,6 @@ class EnvironmentContext(util.ModuleClsProxy):
          .. note:: the ``literal_binds`` flag is ignored on SQLAlchemy
             versions prior to 0.8 where this feature is not supported.
 
-         .. versionadded:: 0.7.6
-
          .. seealso::
 
             :meth:`.Operations.inline_literal`
@@ -420,12 +464,6 @@ class EnvironmentContext(util.ModuleClsProxy):
          only takes effect when the table is first created.
          Defaults to True; setting to False should not be necessary and is
          here for backwards compatibility reasons.
-
-         .. versionadded:: 0.8.10  Added the
-            :paramref:`.EnvironmentContext.configure.version_table_pk`
-            flag and additionally established that the Alembic version table
-            has a primary key constraint by default.
-
         :param on_version_apply: a callable or collection of callables to be
             run for each migration step.
             The callables will be run in the order they are given, once for
@@ -441,9 +479,6 @@ class EnvironmentContext(util.ModuleClsProxy):
               current heads,
             * ``run_args``: the ``**kwargs`` passed to :meth:`.run_migrations`.
 
-            .. versionadded:: 0.9.3
-
-
         Parameters specific to the autogenerate feature, when
         ``alembic revision`` is run with the ``--autogenerate`` feature:
 
@@ -455,14 +490,6 @@ class EnvironmentContext(util.ModuleClsProxy):
          what is locally available on the target
          :class:`~sqlalchemy.engine.Connection`
          to produce candidate upgrade/downgrade operations.
-
-         .. versionchanged:: 0.9.0 the
-            :paramref:`.EnvironmentContext.configure.target_metadata`
-            parameter may now be passed a sequence of
-            :class:`~sqlalchemy.schema.MetaData` objects to support
-            autogeneration of multiple :class:`~sqlalchemy.schema.MetaData`
-            collections.
-
         :param compare_type: Indicates type comparison behavior during
          an autogenerate
          operation.  Defaults to ``False`` which disables type
@@ -521,6 +548,49 @@ class EnvironmentContext(util.ModuleClsProxy):
 
             :paramref:`.EnvironmentContext.configure.compare_type`
 
+        :param include_name: A callable function which is given
+         the chance to return ``True`` or ``False`` for any database reflected
+         object based on its name, including database schema names when
+         the :paramref:`.EnvironmentContext.configure.include_schemas` flag
+         is set to ``True``.
+
+         The function accepts the following positional arguments:
+
+         * ``name``: the name of the object, such as schema name or table name.
+           Will be ``None`` when indicating the default schema name of the
+           database connection.
+         * ``type``: a string describing the type of object; currently
+           ``"schema"``, ``"table"``, ``"column"``, ``"index"``,
+           ``"unique_constraint"``, or ``"foreign_key_constraint"``
+         * ``parent_names``: a dictionary of "parent" object names, that are
+           relative to the name being given.  Keys in this dictionary may
+           include:  ``"schema_name"``, ``"table_name"``.
+
+         E.g.::
+
+            def include_name(name, type_, parent_names):
+                if type_ == "schema":
+                    return name in ["schema_one", "schema_two"]
+                else:
+                    return True
+
+            context.configure(
+                # ...
+                include_schemas = True,
+                include_name = include_name
+            )
+
+         .. versionadded:: 1.5
+
+         .. seealso::
+
+            :ref:`autogenerate_include_hooks`
+
+            :paramref:`.EnvironmentContext.configure.include_object`
+
+            :paramref:`.EnvironmentContext.configure.include_schemas`
+
+
         :param include_object: A callable function which is given
          the chance to return ``True`` or ``False`` for any object,
          indicating if the given object should be considered in the
@@ -539,14 +609,6 @@ class EnvironmentContext(util.ModuleClsProxy):
          * ``type``: a string describing the type of object; currently
            ``"table"``, ``"column"``, ``"index"``, ``"unique_constraint"``,
            or ``"foreign_key_constraint"``
-
-           .. versionadded:: 0.7.0 Support for indexes and unique constraints
-              within the
-              :paramref:`~.EnvironmentContext.configure.include_object` hook.
-
-           .. versionadded:: 0.7.1 Support for foreign keys within the
-              :paramref:`~.EnvironmentContext.configure.include_object` hook.
-
          * ``reflected``: ``True`` if the given object was produced based on
            table reflection, ``False`` if it's from a local :class:`.MetaData`
            object.
@@ -568,52 +630,27 @@ class EnvironmentContext(util.ModuleClsProxy):
                 include_object = include_object
             )
 
-         :paramref:`.EnvironmentContext.configure.include_object` can also
-         be used to filter on specific schemas to include or omit, when
-         the :paramref:`.EnvironmentContext.configure.include_schemas`
-         flag is set to ``True``.   The :attr:`.Table.schema` attribute
-         on each :class:`.Table` object reflected will indicate the name of the
-         schema from which the :class:`.Table` originates.
-
-         .. versionadded:: 0.6.0
+         For the use case of omitting specific schemas from a target database
+         when :paramref:`.EnvironmentContext.configure.include_schemas` is
+         set to ``True``, the :attr:`~sqlalchemy.schema.Table.schema`
+         attribute can be checked for each :class:`~sqlalchemy.schema.Table`
+         object passed to the hook, however it is much more efficient
+         to filter on schemas before reflection of objects takes place
+         using the :paramref:`.EnvironmentContext.configure.include_name`
+         hook.
 
          .. seealso::
 
-            :paramref:`.EnvironmentContext.configure.include_schemas`
+            :ref:`autogenerate_include_hooks`
 
-        :param include_symbol: A callable function which, given a table name
-         and schema name (may be ``None``), returns ``True`` or ``False``,
-         indicating if the given table should be considered in the
-         autogenerate sweep.
-
-         .. deprecated:: 0.6.0
-            :paramref:`.EnvironmentContext.configure.include_symbol`
-            is superceded by the more generic
-            :paramref:`.EnvironmentContext.configure.include_object`
-            parameter.
-
-         E.g.::
-
-            def include_symbol(tablename, schema):
-                return tablename not in ("skip_table_one", "skip_table_two")
-
-            context.configure(
-                # ...
-                include_symbol = include_symbol
-            )
-
-         .. seealso::
+            :paramref:`.EnvironmentContext.configure.include_name`
 
             :paramref:`.EnvironmentContext.configure.include_schemas`
 
-            :paramref:`.EnvironmentContext.configure.include_object`
-
         :param render_as_batch: if True, commands which alter elements
          within a table will be placed under a ``with batch_alter_table():``
          directive, so that batch migrations will take place.
 
-         .. versionadded:: 0.7.0
-
          .. seealso::
 
             :ref:`batch_migrations`
@@ -623,12 +660,16 @@ class EnvironmentContext(util.ModuleClsProxy):
          :meth:`~sqlalchemy.engine.reflection.Inspector.get_schema_names`
          method, and include all differences in tables found across all
          those schemas.  When using this option, you may want to also
-         use the :paramref:`.EnvironmentContext.configure.include_object`
-         option to specify a callable which
+         use the :paramref:`.EnvironmentContext.configure.include_name`
+         parameter to specify a callable which
          can filter the tables/schemas that get included.
 
          .. seealso::
 
+            :ref:`autogenerate_include_hooks`
+
+            :paramref:`.EnvironmentContext.configure.include_name`
+
             :paramref:`.EnvironmentContext.configure.include_object`
 
         :param render_item: Callable that can be used to override how
@@ -694,15 +735,6 @@ class EnvironmentContext(util.ModuleClsProxy):
          in order to future-proof migration files against reorganizations
          in modules.
 
-         .. versionchanged:: 0.7.0
-            :paramref:`.EnvironmentContext.configure.user_module_prefix`
-            no longer defaults to the value of
-            :paramref:`.EnvironmentContext.configure.sqlalchemy_module_prefix`
-            when left at ``None``; the ``__module__`` attribute is now used.
-
-         .. versionadded:: 0.6.3 added
-            :paramref:`.EnvironmentContext.configure.user_module_prefix`
-
          .. seealso::
 
             :ref:`autogen_module_prefix`
@@ -740,18 +772,6 @@ class EnvironmentContext(util.ModuleClsProxy):
          a :class:`.Rewriter` object.  This is a helper object that
          assists in the production of autogenerate-stream rewriter functions.
 
-
-         .. versionadded:: 0.8.0
-
-         .. versionchanged:: 0.8.1 - The
-            :paramref:`.EnvironmentContext.configure.process_revision_directives`
-            hook can append op directives into :class:`.UpgradeOps` and
-            :class:`.DowngradeOps` which will be rendered in Python regardless
-            of whether the ``--autogenerate`` option is in use or not;
-            the ``revision_environment`` configuration variable should be
-            set to "true" in the config to enable this.
-
-
          .. seealso::
 
              :ref:`customizing_revision`
@@ -790,7 +810,7 @@ class EnvironmentContext(util.ModuleClsProxy):
             opts["template_args"].update(template_args)
         opts["transaction_per_migration"] = transaction_per_migration
         opts["target_metadata"] = target_metadata
-        opts["include_symbol"] = include_symbol
+        opts["include_name"] = include_name
         opts["include_object"] = include_object
         opts["include_schemas"] = include_schemas
         opts["render_as_batch"] = render_as_batch
@@ -822,7 +842,7 @@ class EnvironmentContext(util.ModuleClsProxy):
             opts=opts,
         )
 
-    def run_migrations(self, **kw):
+    def run_migrations(self, **kw: Any) -> None:
         """Run migrations as determined by the current command line
         configuration
         as well as versioning information present (or not) in the current
@@ -842,10 +862,15 @@ class EnvironmentContext(util.ModuleClsProxy):
         first been made available via :meth:`.configure`.
 
         """
+        assert self._migration_context is not None
         with Operations.context(self._migration_context):
             self.get_context().run_migrations(**kw)
 
-    def execute(self, sql, execution_options=None):
+    def execute(
+        self,
+        sql: Union[ClauseElement, str],
+        execution_options: Optional[dict] = None,
+    ) -> None:
         """Execute the given SQL using the current change context.
 
         The behavior of :meth:`.execute` is the same
@@ -859,7 +884,7 @@ class EnvironmentContext(util.ModuleClsProxy):
         """
         self.get_context().execute(sql, execution_options=execution_options)
 
-    def static_output(self, text):
+    def static_output(self, text: str) -> None:
         """Emit text directly to the "offline" SQL stream.
 
         Typically this is for emitting comments that
@@ -870,7 +895,9 @@ class EnvironmentContext(util.ModuleClsProxy):
         """
         self.get_context().impl.static_output(text)
 
-    def begin_transaction(self):
+    def begin_transaction(
+        self,
+    ) -> Union[_ProxyTransaction, ContextManager]:
         """Return a context manager that will
         enclose an operation within a "transaction",
         as defined by the environment's offline
@@ -916,7 +943,7 @@ class EnvironmentContext(util.ModuleClsProxy):
 
         return self.get_context().begin_transaction()
 
-    def get_context(self):
+    def get_context(self) -> MigrationContext:
         """Return the current :class:`.MigrationContext` object.
 
         If :meth:`.EnvironmentContext.configure` has not been
@@ -928,7 +955,7 @@ class EnvironmentContext(util.ModuleClsProxy):
             raise Exception("No context has been configured yet.")
         return self._migration_context
 
-    def get_bind(self):
+    def get_bind(self) -> Connection:
         """Return the current 'bind'.
 
         In "online" mode, this is the
@@ -939,7 +966,7 @@ class EnvironmentContext(util.ModuleClsProxy):
         has first been made available via :meth:`.configure`.
 
         """
-        return self.get_context().bind
+        return self.get_context().bind  # type: ignore[return-value]
 
-    def get_impl(self):
+    def get_impl(self) -> DefaultImpl:
         return self.get_context().impl
