@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import collections
 from collections.abc import Iterable
 import textwrap
@@ -28,7 +30,7 @@ _T = TypeVar("_T")
 
 class _ModuleClsMeta(type):
     def __setattr__(cls, key: str, value: Callable) -> None:
-        super(_ModuleClsMeta, cls).__setattr__(key, value)
+        super().__setattr__(key, value)
         cls._update_module_proxies(key)  # type: ignore
 
 
@@ -196,7 +198,7 @@ def to_tuple(x: Any, default: tuple) -> tuple:
 
 
 @overload
-def to_tuple(x: None, default: _T = None) -> _T:
+def to_tuple(x: None, default: Optional[_T] = None) -> _T:
     ...
 
 
@@ -268,7 +270,7 @@ class Dispatcher:
         else:
             return fn_or_list  # type: ignore
 
-    def branch(self) -> "Dispatcher":
+    def branch(self) -> Dispatcher:
         """Return a copy of this dispatcher that is independently
         writable."""
 
@@ -280,3 +282,8 @@ class Dispatcher:
         else:
             d._registry.update(self._registry)
         return d
+
+
+def not_none(value: Optional[_T]) -> _T:
+    assert value is not None
+    return value
